<?php

/**
 * The integration functionality of the plugin.
 *
 * @package    Easy_Translate
 * @subpackage Easy_Translate/admin
 */
class Easy_Translate_API_Integration
{
    /**
     * The slug-name of the settings page on which to show the section.
     */
    const PAGE_NAME = 'admin-easy-translate-integration';

    /**
     * A settings group name.
     */
    const OPTION_GROUP_INTEGRATION = 'easy-translate-integration-group';

    /**
     * Slug-name to identify the section.
     */
    const CREDENTIAL_SECTION_NAME = 'et_api_credentials_section';

    /**
     * Slug-name to identify the section.
     */
    const STRING_LIBRARY_SECTION_NAME = 'et_api_string_library_section';

    /**
     * Slug-names to identify the field.
     */
    const CLIENT_ID_FIELD = 'client_id';
    const CLIENT_SECRET_FIELD = 'client_secret';
    const LOGIN_USERNAME_FIELD = 'login_username';
    const LOGIN_PASSWORD_FIELD = 'login_password';
    const ACCESS_TOKEN_FIELD = 'access_token';
    const REFRESH_TOKEN = 'refresh_token';
    const ACCESS_TOKEN_TTL_FIELD = 'access_token_ttl';
    const CUSTOMER_ID = 'customer_id';
    const COMPANY_NAME = 'company_name';
    const TEAM_IDENTIFIER = 'team_identifier';
    const FIRST_NAME = 'first_name';
    const LAST_NAME = 'last_name';
    const IS_SANDBOX_FIELD = 'is_sandbox';
    const STRING_LIBRARY_ID = 'string_library_id';

    /**
     * Value set for the option.
     */
    private $options;

    private $teams;

    /**
     * Easy_Translate_API_Integration constructor.
     */
    public function __construct()
    {
        $this->options = get_option(EASY_TRANSLATE_INTEGRATION_OPTION);
        $this->teams = [];
    }

    /**
     * A callback function that sanitizes the option's value.
     *
     * @param $input
     * @return array
     */
    public function sanitize_before_save($input): array
    {
        $newInput = [];

        foreach ($input as $id => $value) {
            $newInput[$id] = sanitize_text_field($value);
        }
        if (empty($newInput[self::LOGIN_PASSWORD_FIELD]) && ($newInput[self::IS_SANDBOX_FIELD] === $this->options[self::IS_SANDBOX_FIELD])) {
            return array_merge($this->options, $newInput);
        }

        $service = new Easy_Translate_API_Laas($this->map_fields($newInput));
        $response = $service->login();

        if ($response['error'] ?? false) {
            add_settings_error(
                self::ACCESS_TOKEN_FIELD,
                'access-token-error',
                $response['error'] ?? '',
                'error'
            );
            add_settings_error(
                self::ACCESS_TOKEN_FIELD,
                'access-token-error',
                $response['hint'] ?? '',
                'info'
            );
            $newInput[self::ACCESS_TOKEN_FIELD] = '';
            $newInput[self::REFRESH_TOKEN] = '';
            $newInput[self::ACCESS_TOKEN_TTL_FIELD] = '';
            $newInput[self::CUSTOMER_ID] = '';
            $newInput[self::COMPANY_NAME] = '';
            $newInput[self::TEAM_IDENTIFIER] = '';
            $newInput[self::FIRST_NAME] = '';
            $newInput[self::LAST_NAME] = '';
            $newInput[self::STRING_LIBRARY_ID] = '';

            return $newInput;
        }
        $newInput[self::ACCESS_TOKEN_FIELD] = $response['access_token'];
        $newInput[self::REFRESH_TOKEN] = $response['refresh_token'];
        $newInput[self::ACCESS_TOKEN_TTL_FIELD] = date(
            'Y-m-d H:i:s',
            strtotime('now') + $response['expires_in']
        );

        if (!empty($newInput[self::ACCESS_TOKEN_FIELD] ?? false)) {
            $customer = $service->logged_customer();
            $newInput[self::CUSTOMER_ID] = $customer['data']['id'];
            $newInput[self::FIRST_NAME] = $customer['data']['attributes']['first_name'];
            $newInput[self::LAST_NAME] = $customer['data']['attributes']['last_name'];
            $newInput[self::COMPANY_NAME] = $customer['included'][0]['attributes']['company_name'];
            $newInput[self::TEAM_IDENTIFIER] = $customer['included'][0]['attributes']['team_identifier'];
        }

        $newInput[self::LOGIN_PASSWORD_FIELD] = '';

        return $newInput;
    }

    /**
     * Adds a section to the page.
     */
    public function register_credentials_section()
    {
        add_settings_section(
            self::CREDENTIAL_SECTION_NAME,
            'Enter your login credentials:',
            [],
            self::PAGE_NAME
        );
        if ($this->options[self::ACCESS_TOKEN_FIELD] ?? false) {
            add_settings_section(
                self::STRING_LIBRARY_SECTION_NAME,
                'Choose your string library:',
                [],
                self::PAGE_NAME
            );
        }

        $this->showFields();
    }

    /**
     * Adds fields to the section.
     */
    public function showFields()
    {
        add_settings_field(
            self::CLIENT_ID_FIELD,
            __('Client ID', EASY_TRANSLATE),
            [$this, 'client_id_handler'],
            self::PAGE_NAME,
            self::CREDENTIAL_SECTION_NAME
        );
        add_settings_field(
            self::CLIENT_SECRET_FIELD,
            __('Client Secret', EASY_TRANSLATE),
            [$this, 'client_secret_handler'],
            self::PAGE_NAME,
            self::CREDENTIAL_SECTION_NAME
        );
        add_settings_field(
            self::LOGIN_USERNAME_FIELD,
            __('Username', EASY_TRANSLATE),
            [$this, 'username_handler'],
            self::PAGE_NAME,
            self::CREDENTIAL_SECTION_NAME
        );
        add_settings_field(
            self::LOGIN_PASSWORD_FIELD,
            __('Password', EASY_TRANSLATE),
            [$this, 'password_handler'],
            self::PAGE_NAME,
            self::CREDENTIAL_SECTION_NAME
        );
        add_settings_field(
            self::ACCESS_TOKEN_FIELD,
            __('Access Token', EASY_TRANSLATE),
            [$this, 'access_token'],
            self::PAGE_NAME,
            self::CREDENTIAL_SECTION_NAME
        );
        add_settings_field(
            self::ACCESS_TOKEN_TTL_FIELD,
            __('Access Token Valid Until', EASY_TRANSLATE),
            [$this, 'access_token_ttl'],
            self::PAGE_NAME,
            self::CREDENTIAL_SECTION_NAME
        );
        if ($this->options[self::ACCESS_TOKEN_FIELD] ?? false) {
            add_settings_field(
                self::TEAM_IDENTIFIER,
                __('Team', EASY_TRANSLATE),
                [$this, 'team'],
                self::PAGE_NAME,
                self::CREDENTIAL_SECTION_NAME
            );
        }
        add_settings_field(
            self::IS_SANDBOX_FIELD,
            __('Sandbox mode', EASY_TRANSLATE),
            [$this, 'is_sandbox_handler'],
            self::PAGE_NAME,
            self::CREDENTIAL_SECTION_NAME
        );
        add_settings_field(
            self::STRING_LIBRARY_ID,
            __('String Library ID', EASY_TRANSLATE),
            [$this, 'string_library_id_handler'],
            self::PAGE_NAME,
            self::STRING_LIBRARY_SECTION_NAME
        );
    }

    /**
     * Show the client ID field.
     */
    public function client_id_handler()
    {
        $this->text_field(self::CLIENT_ID_FIELD, __('Client ID from our platform.', EASY_TRANSLATE));
    }

    /**
     * Show the client secret field.
     */
    public function client_secret_handler()
    {
        $this->text_field(self::CLIENT_SECRET_FIELD, __('Client Secret from our platform.', EASY_TRANSLATE));
    }

    /**
     * Show the username field.
     */
    public function username_handler()
    {
        $this->text_field(self::LOGIN_USERNAME_FIELD, __('Login Username from our platform.', EASY_TRANSLATE));
    }

    /**
     * Show the password field.
     */
    public function password_handler()
    {
        $this->text_field(self::LOGIN_PASSWORD_FIELD, __('Login Password from our platform.', EASY_TRANSLATE));
    }

    /**
     * Show the access token field.
     */
    public function access_token()
    {
        if ($this->options[self::ACCESS_TOKEN_FIELD] ?? false) {
            echo "<p><strong>" . __('Access token is set properly.', EASY_TRANSLATE) . "</strong></p>";

            return;
        }

        echo "<p class=\"notice notice-error settings-error\"><span><strong>" . __('Access token is not yet set.',
                EASY_TRANSLATE) . "</strong></span></p>";
    }

    /**
     * Show the access token TTL field.
     */
    public function access_token_ttl()
    {
        if ($this->options[self::ACCESS_TOKEN_TTL_FIELD] ?? false) {
            echo "<p><strong>{$this->options[self::ACCESS_TOKEN_TTL_FIELD]}</strong></p>";

            return;
        }

        echo "<p class=\"notice notice-error settings-error\"><strong>" . __('Access token is not yet set.',
                EASY_TRANSLATE) . "</strong></p>";
    }

    /**
     * Show the access token TTL field.
     */
    public function team()
    {
        ?>
      <select id="<?= self::TEAM_IDENTIFIER ?>"
              name="<?= EASY_TRANSLATE_INTEGRATION_OPTION ?>[<?= self::TEAM_IDENTIFIER ?>]">
          <?php foreach ($this->teams as $team) { ?>
            <option
              value="<?= $team['attributes']['team_identifier'] ?>" <?= ($this->options[self::TEAM_IDENTIFIER] === $team['attributes']['team_identifier']) ? 'selected' : '' ?>
            ><?= $team['attributes']['company_name'] ?> (<?= $team['attributes']['team_identifier'] ?>)
            </option>
          <?php } ?>
      </select>
        <?php
    }

    /**
     * Show the sandbox mode field.
     */
    public function is_sandbox_handler()
    {
        $id = self::IS_SANDBOX_FIELD;
        $checked = isset($this->options[$id]) && $this->options[$id] === '1' ? 'checked' : '';
        printf(
            "<input type=\"checkbox\" id=\"{$id}\" name=\"" . EASY_TRANSLATE_INTEGRATION_OPTION . "[{$id}]\" value=\"1\"/ %s>",
            $checked
        );
    }

    public function string_library_id_handler()
    {
        $this->text_field(self::STRING_LIBRARY_ID, __('String Library For Your WPML', EASY_TRANSLATE));
        $librariesUrl = "https://platform.easytranslate.com/en/customer/{$this->options[self::TEAM_IDENTIFIER]}/libraries";
        if ($libraryId = ($this->options[self::STRING_LIBRARY_ID] ?? false)) {
            $libraryUrl = "{$librariesUrl}/{$libraryId}";
            echo '<br>Your library: <a href="' . $libraryUrl . '">' . $libraryUrl . '</a>';
        } else {
            echo '<br>You can create your WPML library here: <a href="' . $librariesUrl . '">' . $librariesUrl . '</a>';
        }
    }

    /**
     * Base wrapper for the text fields.
     *
     * @param string $id
     * @param string $placeholder
     */
    private function text_field(string $id, string $placeholder)
    {
        printf(
            "<input type=\"text\" id=\"{$id}\" name=\"" . EASY_TRANSLATE_INTEGRATION_OPTION . "[{$id}]\" value=\"%s\" placeholder=\"{$placeholder}\"/>",
            esc_attr($this->options[$id] ?? '')
        );
    }

    /**
     * Returns the array of mapped option fields.
     *
     * @param array $options
     * @return array
     */
    public static function map_fields(array $options): array
    {
        return [
            'client_id'         => $options[self::CLIENT_ID_FIELD] ?? null,
            'client_secret'     => $options[self::CLIENT_SECRET_FIELD] ?? null,
            'username'          => $options[self::LOGIN_USERNAME_FIELD] ?? null,
            'password'          => $options[self::LOGIN_PASSWORD_FIELD] ?? null,
            'access_token'      => $options[self::ACCESS_TOKEN_FIELD] ?? null,
            'refresh_token'     => $options[self::REFRESH_TOKEN] ?? null,
            'customer_id'       => $options[self::CUSTOMER_ID] ?? null,
            'company_name'      => $options[self::COMPANY_NAME] ?? null,
            'team_identifier'   => $options[self::TEAM_IDENTIFIER] ?? null,
            'first_name'        => $options[self::FIRST_NAME] ?? null,
            'last_name'         => $options[self::LAST_NAME] ?? null,
            'is_sandbox'        => $options[self::IS_SANDBOX_FIELD] ?? null,
            'string_library_id' => $options[self::STRING_LIBRARY_ID] ?? null,
        ];
    }

    /**
     * Include API integration Partial.
     */
    public function include_api_integration_partial()
    {
        include_once(EASY_TRANSLATE_ROOT . 'admin/partials/api-integration.php');
    }

    /**
     * Register the form fields for the API integration page.
     */
    public function init()
    {
        register_setting(
            self::OPTION_GROUP_INTEGRATION,
            EASY_TRANSLATE_INTEGRATION_OPTION,
            [$this, 'sanitize_before_save']
        );
        if (!empty($this->options[self::ACCESS_TOKEN_FIELD] ?? false)) {
            $service = new Easy_Translate_API_Laas($this->options);
            $loggedUser = $service->logged_customer();
            if (empty($loggedUser['included'] ?? [])) {
                add_settings_error(
                    self::ACCESS_TOKEN_FIELD,
                    'access-token-error',
                    "Something went wrong with fetching the user. Response: " . json_encode($loggedUser),
                    'error'
                );
            }
            foreach ($loggedUser['included'] ?? [] as $included) {
                if ($included['type'] === 'account') {
                    $this->teams[] = $included;
                }
            }
        }

        $this->register_credentials_section();
    }
}
