<?php

/**
 * The API Service class.
 *
 * @package    Easy_Translate
 * @subpackage Easy_Translate/admin
 */
abstract class Easy_Translate_API_Service
{
    /**
     * The dynamic portion of the hook name, $action, refers to the name of the Ajax action callback being fired.
     */
    const WEB_HOOK_ACTION = 'et_api_web_hook_action';

    /**
     * API base url for production.
     */
    const BASE_URL_PRODUCTION = 'https://api.platform.easytranslate.com';

    /**
     * API base url for sandbox.
     */
    const BASE_URL_SANDBOX = 'https://api.platform.sandbox.easytranslate.com';

    /**
     * API version.
     */
    const API_VERSION = 'v1';

    /**
     * Type of grant requested.
     */
    const GRANT_TYPE_PASSWORD = 'password';

    /**
     * Scopes for the User.
     */
    const SCOPE_DASHBOARD = 'dashboard';

    /**
     * Client ID provided by EasyTranslate.
     */
    private $client_id;

    /**
     * Client secret provided by EasyTranslate.
     */
    private $client_secret;

    /**
     * Username/email of the User who will perform the actions.
     */
    private $username;

    /**
     * Password of the User who will perform the actions.
     */
    private $password;

    /**
     * The access token for API authentication.
     */
    private $access_token = null;

    /**
     * The refresh token for API authentication.
     */
    private $refresh_token = null;

    /**
     * The ID of the customer that is logged in.
     */
    private $customer_id;

    /**
     * The team ID of the customer that is logged in.
     */
    private $team_identifier;

    /**
     * The string library ID used to sync strings
     */
    private $string_library_id;

    /**
     *  The sandbox mode flag.
     */
    private $is_sandbox_mode = null;

    /**
     * Easy_Translate_API_Service constructor.
     *
     * @param array $options
     */
    public function __construct(array $options)
    {
        $this->set_initial_values($options);
    }

    /**
     * Obtain the access token for further authentications.
     *
     * @return array
     */
    public function login(): array
    {
        $response = $this->wp_remote_post('/oauth/token', ['body' => $this->login_credentials()]);
        $httpCode = wp_remote_retrieve_response_code($response);
        $body = $this->decode_body($response);

        if ($httpCode != 200) {
            return [
                'error' => isset($body['message']) && $body['message'] ? $body['message'] : __('Error', EASY_TRANSLATE),
                'hint'  => isset($body['hint']) && $body['hint'] ? $body['hint'] : __('Please verify your credentials',
                    EASY_TRANSLATE),
            ];
        }

        $this->access_token = $body['access_token'];

        return $body;
    }

    /**
     * Get the currently authenticated user.
     *
     * @return array
     */
    public function logged_customer(): array
    {
        $path = $this->versioned_path('user');
        $response = $this->wp_remote_get($path);

        return $this->decode_body($response);
    }

    protected function get_string_library_id()
    {
        return $this->string_library_id;
    }

    /**
     * Wrapper method for Wordpress function wp_remote_get.
     *
     * @param $url
     * @return array|WP_Error
     */
    protected function wp_remote_get($path)
    {
        $url = $this->get_base_url() . $path;

        return $this->wp_remote_get_raw($url);
    }

    /**
     * Wrapper method for WordPress function wp_remote_get.
     *
     * @param $url
     * @return array|WP_Error
     */
    protected function wp_remote_get_raw($url)
    {
        $headers = ['Accept' => 'application/json'];
        if ($this->access_token !== null) {
            $headers['Authorization'] = "Bearer {$this->access_token}";
        }
        $args = ['headers' => $headers];
        $args['timeout'] = 300;

        return wp_remote_get($url, $args);
    }

    /**
     * Wrapper method for Wordpress function wp_remote_post.
     *
     * @param $path
     * @param $args
     * @return array|WP_Error
     */
    protected function wp_remote_post($path, $args)
    {
        $url = $this->get_base_url() . $path;
        $headers = [
            'Accept'       => 'application/json',
            'Content-Type' => 'application/x-www-form-urlencoded',
        ];
        $args['headers'] = array_merge($headers, $args['headers'] ?? []);

        if ($this->access_token !== null) {
            $args['headers']['Authorization'] = "Bearer {$this->access_token}";
        }
        $args['timeout'] = 300;

        return wp_remote_post($url, $args);
    }

    /**
     * Returns an array of the decoded response body.
     *
     * @param $response
     * @return null|array
     */
    protected function decode_body($response)
    {
        if ($response instanceof WP_Error) {
            return $response->get_error_messages();
        }

        return json_decode((string)$response['body'], true);
    }

    /**
     * Returns path with current API version.
     *
     * @param string $path
     * @return string
     */
    protected function versioned_path(string $path): string
    {
        return "/{$this->get_service_prefix()}/api/" . self::API_VERSION . "/{$path}";
    }

    /**
     * Returns the path with current API version for the selected team
     *
     * @param string $path
     * @return string
     */
    protected function team_path(string $path): string
    {
        return $this->versioned_path("teams/{$this->team_identifier}/{$path}");
    }

    /**
     * Converts languages to the EasyTranslate standard.
     *
     * @param $source
     * @return string
     */
    protected function normalize_source($source): string
    {
        return get_et_source_language_code($source);
    }

    /**
     * Converts languages to the EasyTranslate standard.
     *
     * @param $target
     * @return array|string|string[]
     */
    protected function normalize_target($target)
    {
        if (!is_array($target)) {
            return get_et_target_language_code($target);
        } else {
            return array_map(function (string $language) {
                return get_et_target_language_code($language);
            }, $target);
        }
    }

    /**
     * Returns array with login credentials.
     *
     * @return array
     */
    private function login_credentials(): array
    {
        return [
            'client_id'     => $this->client_id,
            'client_secret' => $this->client_secret,
            'username'      => $this->username,
            'password'      => $this->password,
            'grant_type'    => self::GRANT_TYPE_PASSWORD,
            'scope'         => self::SCOPE_DASHBOARD,
        ];
    }

    /**
     * Set the credentials/login details.
     *
     * @param array $options
     */
    private function set_initial_values(array $options = [])
    {
        $this->client_id = $options['client_id'] ?? null;
        $this->client_secret = $options['client_secret'] ?? null;
        $this->username = $options['username'] ?? null;
        $this->password = $options['password'] ?? null;
        $this->access_token = $options['access_token'] ?? null;
        $this->refresh_token = $options['refresh_token'] ?? null;
        $this->customer_id = $options['customer_id'] ?? null;
        $this->team_identifier = $options['team_identifier'] ?? null;
        $this->string_library_id = $options['string_library_id'] ?? null;
        $this->is_sandbox_mode = $options['is_sandbox'] ?? null;
    }

    /**
     * Returns API base url.
     *
     * @return string
     */
    private function get_base_url(): string
    {
        if ($this->is_sandbox_mode === '1') {
            return self::BASE_URL_SANDBOX;
        }

        return self::BASE_URL_PRODUCTION;
    }

    abstract protected function get_service_prefix(): string;
}
