<?php

/**
 * Class Easy_Translate_String_Translation_List.
 *
 * @package    Easy_Translate
 * @subpackage Easy_Translate/admin
 */
class Easy_Translate_String_Translation_List extends Easy_Translate_Translation_List
{
    /**
     * Per page item option.
     */
    const PER_PAGE_OPTION = 'edit_string_translation_per_page';

    /**
     * Easy_Translate_String_Translation_List constructor.
     */
    public function __construct()
    {
        parent::__construct([
            'singular' => __('translation', EASY_TRANSLATE),
            'plural'   => __('translations', EASY_TRANSLATE),
            'ajax'     => false,
        ]);
    }

    /**
     * Get a list of columns.
     *
     * @return array
     */
    public function get_columns()
    {

        $table_columns = [
            'cb'              => '<input type="checkbox" />', // to display the checkbox.
            'context'         => __('Domain', EASY_TRANSLATE),
            'gettext_context' => __('Context', EASY_TRANSLATE),
            'page'            => __('Page', EASY_TRANSLATE),
            'value'           => __('String', EASY_TRANSLATE),
            'status'          => __('Status', EASY_TRANSLATE),
        ];

        foreach ($this->translation_languages as $lang) {
            $table_columns["{$lang}_value"] = __('Translation', EASY_TRANSLATE) . " ({$lang})";
        }

        return $table_columns;

    }

    /**
     * Get a list of sortable columns.
     *
     * @return array
     */
    protected function get_sortable_columns()
    {

        $sortable_columns = [
            'context' => 'context',
            'status'  => 'status',
        ];

        return $sortable_columns;

    }

    /**
     * Text is displayed when there are no translated strings.
     */
    public function no_items()
    {

        _e('No translated strings available.', EASY_TRANSLATE);

    }

    /**
     * Fetch table data from the WordPress database.
     *
     * @return array|object|null
     */
    public function fetch_table_data()
    {

        global $wpdb;

        $wpdb_table = $wpdb->prefix . 'icl_strings';
        $orderby = (isset($_GET['orderby'])) ? esc_sql($_GET['orderby']) : 'id';
        $order = (isset($_GET['order'])) ? esc_sql($_GET['order']) : 'DESC';

        $a_where = [];
        $where_query = '';
        if (isset($_GET['string_status']) && $_GET['string_status'] !== '') {
            $a_where[] = $wpdb_table . '.status=' . esc_sql($_GET['string_status']);
        }
        if (isset($_GET['domain']) && $_GET['domain'] !== '') {
            $a_where[] = $wpdb_table . '.context="' . esc_sql($_GET['domain'] == 'empty-context-domain' ? '' : $_GET['domain']) . '"';
        }
        if (count($a_where) > 0) {
            $where_query = 'WHERE ' . implode(' AND ', $a_where);
        }

        $a_select = [];
        $a_join = [];
        $a_select[] = "{$wpdb_table}.context";
        $a_select[] = "{$wpdb_table}.gettext_context";
        $a_select[] = "{$wpdb_table}.value";
        $a_select[] = "{$wpdb_table}.status";
        $a_select[] = "{$wpdb_table}.language";
        $a_select[] = "{$wpdb_table}.id";
        $a_select[] = "{$wpdb_table}.string_package_id";
        $a_select[] = "{$wpdb->prefix}icl_string_packages.title as string_package_title";
        $a_select[] = "{$wpdb->prefix}posts.id as post_id";
        $a_select[] = "{$wpdb->prefix}posts.post_title";
        foreach ($this->translation_languages as $lang) {
            $a_select[] = "`{$lang}`.value AS `{$lang}_value`, `{$lang}`.mo_string AS `{$lang}_string`";
            $a_join[] = "LEFT JOIN {$wpdb->prefix}icl_string_translations AS `{$lang}` ON `{$lang}`.string_id={$wpdb_table}.id AND `{$lang}`.language='{$lang}'";
        }

        $user_query = "SELECT " . implode(', ', $a_select) . " FROM {$wpdb_table} "
                      . " LEFT JOIN {$wpdb->prefix}icl_string_packages ON {$wpdb_table}.string_package_id = {$wpdb->prefix}icl_string_packages.ID "
                      . " LEFT JOIN {$wpdb->prefix}posts ON {$wpdb->prefix}icl_string_packages.post_id = {$wpdb->prefix}posts.ID "
                      . implode(' ', $a_join) . "
						{$where_query} ORDER BY {$wpdb_table}.{$orderby} {$order}";

        // query output_type will be an associative array with ARRAY_A.
        $query_results = $wpdb->get_results($user_query, ARRAY_A);

        // return result array to prepare_items.
        return $query_results;

    }

    /**
     * Render a column when no column specific method exists.
     *
     * @param array  $item
     * @param string $column_name
     * @return mixed
     */
    public function column_default($item, $column_name)
    {
        switch ($column_name) {
            case 'status':
                return $this->get_string_status_label($item[$column_name]);
            case 'value':
                return '(' . $item['language'] . ')&nbsp;' . $item[$column_name];
            case (preg_match('/_value/', $column_name) ? true : false):
                $lang = explode('_', $column_name)[0];

                return $item[$column_name] ? $item[$column_name] : $item["{$lang}_string"];
            default:
                return $item[$column_name];
        }
    }

    /**
     * Get value for checkbox column.
     *
     * @param object $item A row's data
     * @return string Text to be placed inside the column <td>.
     */
    protected function column_cb($item)
    {
        return sprintf(
            "<input type='checkbox' name='strings[]' id='string_{$item['id']}' value='{$item['id']}' />"
        );
    }

    /**
     * Get value for page column.
     *
     * @param object $item A row's data
     * @return string Text to be placed inside the column <td>.
     */
    protected function column_page($item)
    {

        if (isset($item['post_id']) && $item['post_id']) {
            return '<a href="' . get_site_url() . '/wp-admin/post.php?post=' . $item['post_id'] . '&action=edit">' . $item['post_title'] . '</a>';
        }

        return null;

    }

    /**
     * Process actions triggered by the user
     */
    public function handle_table_actions()
    {
        /*
         * Note: Table bulk_actions can be identified by checking $_POST['action'] and $_POST['action2']
         *
         * action - is set if checkbox from top-most select-all is set, otherwise returns -1
         * action2 - is set if checkbox the bottom-most select-all checkbox is set, otherwise returns -1
         */
        if ('sync_to_string_library' === ($_POST['action'] ?? null)) {
            $nonce = wp_unslash($_POST['_wpnonce']);
            if (!wp_verify_nonce($nonce, 'bulk-translations')) {
                $this->invalid_nonce_redirect();

                return null;
            }
            $stringIds = $_POST['strings'] ?? [];

            // Sync with String Library
            // Prepare the strings
            $options = get_option(EASY_TRANSLATE_INTEGRATION_OPTION);
            $libraryApi = new Easy_Translate_API_String_Library(Easy_Translate_API_Integration::map_fields($options));
            $response = $libraryApi->sync_to_string_library(
                self::get_source_language(),
                self::get_target_languages(),
                $this->get_source_content($stringIds)
            );
            if ($response['error'] ?? false) {
                add_settings_error(
                    '',
                    'translate-error',
                    $response['error'] ?? '',
                    'error'
                );

                return null;
            }

            return null;
        }
        if ('sync_from_string_library' === ($_POST['action'] ?? null)) {
            $nonce = wp_unslash($_POST['_wpnonce']);
            if (!wp_verify_nonce($nonce, 'bulk-translations')) {
                $this->invalid_nonce_redirect();

                return null;
            }
            $translationKeys = [];
            $stringIds = $_POST['strings'] ?? [];

            // Sync with String Library
            // Prepare the strings
            $options = get_option(EASY_TRANSLATE_INTEGRATION_OPTION);
            $libraryApi = new Easy_Translate_API_String_Library(Easy_Translate_API_Integration::map_fields($options));
            if (!empty($stringIds)) {
                $content = $this->get_source_content($stringIds);
                $translationKeys = array_map(function (array $translation) use ($libraryApi) {
                    return $libraryApi->buildKey($translation['name'], $translation['context']);
                }, $content);
            }

            $response = $libraryApi->auto_paginated_translations(self::get_target_languages(), $translationKeys);
            if (is_array($response) && ($response['error'] ?? false)) {
                add_settings_error(
                    '',
                    'translate-error',
                    $response['error'] ?? '',
                    'error'
                );

                return null;
            }

            global $wpdb;
            $sourceLanguage = self::get_source_language();
            foreach ($response as $key) {
                $translations = $key['attributes']['translations'];
                $sourceTranslation = array_filter($translations, function (array $translation) use ($sourceLanguage) {
                    return get_wpml_source_language_code($translation['language_code']) === $sourceLanguage;
                })[0];
                foreach ($key['attributes']['translations'] as $translation) {
                    if (($translation['language_code'] === $sourceTranslation['language_code'])) {
                        continue;
                    }
                    $text = $wpdb->_real_escape($translation['text'] ?? null);
                    $targetLanguage = get_wpml_target_language_code($translation['language_code']);
                    $wpmlString = new WPML_ST_String($sourceTranslation['segment_id'], $wpdb);
                    $wpmlString->set_translation(
                        $targetLanguage,
                        $text,
                        empty($text) ? ICL_TM_NOT_TRANSLATED : ICL_TM_COMPLETE
                    );
                }
            }

            return null;
        }

        // check for table bulk actions
        if ('translate_strings' === ($_POST['action'] ?? null) && isset($_POST['strings']) && isset($_POST['target_languages'])) {
            $nonce = wp_unslash($_POST['_wpnonce']);
            if (!wp_verify_nonce($nonce, 'bulk-translations')) {
                $this->invalid_nonce_redirect();

                return null;
            }

            $content = $this->get_strings_content($_POST['strings'], $_POST['source_language'] ?? null);

            $workflow = 'translation';
            if ($_POST['workflow'] === 'self+machine_translation') {
                $workflow = 'self+machine_translation';
            }

            $urlParts = parse_url(home_url());
            $hostName = $urlParts['host'] ?? get_site_url();
            $name = '[' . date('Y-m-d') . '] ' . $hostName . ' string translations';

            $options = get_option(EASY_TRANSLATE_INTEGRATION_OPTION);
            $service = new Easy_Translate_API_Laas(Easy_Translate_API_Integration::map_fields($options));
            $response = $service->translate(
                $_REQUEST['source_language'] ?? $this->source_language,
                array_keys($_POST['target_languages']),
                [$content],
                $workflow,
                $name
            );

            if ($response['error'] ?? false) {
                add_settings_error(
                    '',
                    'translate-error',
                    $response['error'] ?? '',
                    'error'
                );

                return null;
            }

            if ($this->update_string_status($_POST['strings'], self::STATUS_WAITING) !== false) {
                add_settings_error(
                    '',
                    'translate-success',
                    __('Sent successfully!', EASY_TRANSLATE),
                    'success'
                );
            }
        }
    }

    /**
     * Register and configure list table screen option.
     */
    public function load_screen_options()
    {
        add_screen_option('per_page', [
            'label'   => __('Strings per page', EASY_TRANSLATE),
            'default' => 20,
            'option'  => self::PER_PAGE_OPTION,
        ]);
    }

    /**
     * Extra controls to be displayed between bulk actions and pagination.
     *
     * @param string $which
     */
    protected function extra_tablenav($which)
    {
        global $wpdb;

        if ($which == 'top') {
            $current_status = isset($_GET['string_status']) ? $_GET['string_status'] : '';
            ?>
          <label for="string_status"><?php echo __('Select which strings to display: ', EASY_TRANSLATE) ?></label>
          <select name="string_status" id="string_status">
            <option <?php selected($current_status, ''); ?>
              value=""><?php echo __('All strings', EASY_TRANSLATE) ?></option>
            <option <?php selected($current_status, self::STATUS_COMPLETE); ?>
              value="<?= self::STATUS_COMPLETE ?>"><?php echo $this->get_string_status_label(self::STATUS_COMPLETE) ?></option>
            <option <?php selected($current_status, self::STATUS_NOT_TRANSLATED); ?>
              value="<?= self::STATUS_NOT_TRANSLATED ?>"><?php echo $this->get_string_status_label(self::STATUS_NOT_TRANSLATED) ?></option>
            <option <?php selected($current_status, self::STATUS_PARTIAL); ?>
              value="<?= self::STATUS_PARTIAL ?>"><?php echo $this->get_string_status_label(self::STATUS_PARTIAL) ?></option>
            <option <?php selected($current_status, self::STATUS_WAITING); ?>
              value="<?= self::STATUS_WAITING ?>"><?php echo $this->get_string_status_label(self::STATUS_WAITING) ?></option>
            <option <?php selected($current_status, self::STATUS_NEEDS_UPDATE); ?>
              value="<?= self::STATUS_NEEDS_UPDATE ?>"><?php echo $this->get_string_status_label(self::STATUS_NEEDS_UPDATE) ?></option>
          </select>


            <?php
            $current_domain = $_GET['domain'] ?? '';
            $domains = $wpdb->get_results('SELECT context , COUNT(context) as count_domain FROM ' . $wpdb->prefix . 'icl_strings GROUP BY context',
                ARRAY_A);
            ?>

          <label for="domain"><?php echo __('Select strings within domain: ', EASY_TRANSLATE) ?></label>
          <select name="domain" id="domain">
            <option <?php selected($current_domain, ''); ?>
              value=""><?php echo __('All domains', EASY_TRANSLATE) ?></option>
              <?php
              foreach ($domains as $domain) {
                  $context = $domain['context'] != '' ? $domain['context'] : 'empty-context-domain';
                  ?>
                <option <?php selected($current_domain, $context); ?> value="<?= $context ?>"><?= $context ?>
                  (<?= $domain['count_domain'] ?>)
                </option>
                  <?php
              }
              ?>

          </select>

            <?php
            submit_button(__('Filter', EASY_TRANSLATE), '', '', false);
        }

    }

    /**
     * Render the list table.
     */
    protected function render_list()
    {
        include_once(EASY_TRANSLATE_ROOT . 'admin/partials/string-translation.php');
    }

    /**
     * Returns per page option.
     *
     * @return string
     */
    protected function get_per_page_option()
    {
        return self::PER_PAGE_OPTION;
    }

    private function get_source_content(array $stringIds = []): array
    {
        global $wpdb;
        $prefix = $wpdb->prefix;
        $query = "select s.id      as source_id,
                     s.language    as source_language,
                     s.context,
                     s.name,
                     s.value       as source_text,
                     wist.id       as target_id,
                     wist.value    as target_text,
                     wist.language as target_language
              from {$prefix}icl_strings s
              left join {$prefix}icl_string_translations wist on wist.string_id = s.id";
        if (!empty($stringIds)) {
            $stringIdsQuery = implode(',', $stringIds);
            $query .= " where s.id in ($stringIdsQuery)";
        }

        return $wpdb->get_results($query, ARRAY_A);
    }
}
