<?php

/**
 * Class Easy_Translate_Translation_List.
 *
 * @package    Easy_Translate
 * @subpackage Easy_Translate/admin
 */
abstract class Easy_Translate_Translation_List extends WP_List_Table
{
    const YOAST_SEO_FIELDS = [
        '_yoast_wpseo_opengraph-description',
        '_yoast_wpseo_opengraph-title',
        '_yoast_wpseo_twitter-description',
        '_yoast_wpseo_twitter-title',
        '_yoast_wpseo_focuskw',
        '_yoast_wpseo_title',
        '_yoast_wpseo_metadesc',
        '_yoast_wpseo_bctitle',
    ];
    /**
     * Translation statuses.
     */
    const STATUS_NOT_TRANSLATED = ICL_TM_NOT_TRANSLATED;
    const STATUS_WAITING = ICL_TM_WAITING_FOR_TRANSLATOR;
    const STATUS_PARTIAL = ICL_STRING_TRANSLATION_PARTIAL;
    const STATUS_NEEDS_UPDATE = ICL_TM_NEEDS_UPDATE;
    const STATUS_DUPLICATE = ICL_TM_DUPLICATE;
    const STATUS_COMPLETE = ICL_TM_COMPLETE;

    /**
     * Sitepress settings option.
     */
    const ICL_SITEPRESS_SETTINGS = 'icl_sitepress_settings';

    /**
     * Source language.
     *
     * @var string
     */
    public $source_language;

    /**
     * Translation languages.
     *
     * @var array
     */
    public $translation_languages = [];

    /**
     * The variable is set to true if the translation functionality is ready to run.
     *
     * @var boolean
     */
    protected $is_ready;

    /**
     * The list of dependency messages.
     *
     * @var array
     */
    protected $dependency_message_list = [];

    /**
     * Easy_Translate_Translation_List constructor.
     *
     * @param array $params
     */
    public function __construct($params)
    {
        $this->is_ready = self::check_dependencies();
        if (!$this->is_ready) {
            $this->dependency_message_list = $this->get_dependency_message_list();
        }

        $this->source_language = self::get_source_language();

        if ($this->is_ready) {
            $this->translation_languages = self::get_target_languages();
        }

        parent::__construct($params);
    }

    /**
     * Prepares the list of items for displaying.
     * Query, filter data, handle sorting, and pagination, and any other data-manipulation required prior to rendering
     */
    public function prepare_items()
    {
        if (!$this->is_ready) {
            return null;
        }

        // check if a search was performed.
        $search_key = isset($_REQUEST['s']) ? wp_unslash(trim($_REQUEST['s'])) : '';

        $this->_column_headers = $this->get_column_info();

        // check and process any actions such as bulk actions.
        $this->handle_table_actions();

        // fetch table data
        $table_data = $this->fetch_table_data();
        // filter the data in case of a search.
        if ($search_key) {
            $table_data = $this->filter_table_data($table_data, $search_key);
        }

        // required for pagination
        $strings_per_page = $this->get_items_per_page($this->get_per_page_option());
        $table_page = $this->get_pagenum();

        // provide the ordered data to the List Table.
        // we need to manually slice the data based on the current pagination.
        $this->items = array_slice($table_data, (($table_page - 1) * $strings_per_page), $strings_per_page);

        // set the pagination arguments
        $total_items = count($table_data);
        $this->set_pagination_args([
            'total_items' => $total_items,
            'per_page'    => $strings_per_page,
            'total_pages' => ceil($total_items / $strings_per_page),
        ]);
    }

    /**
     * Filter the table data based on the user search key.
     *
     * @param $table_data
     * @param $search_key
     * @return array
     */
    public function filter_table_data($table_data, $search_key)
    {

        $filtered_table_data = array_values(array_filter($table_data, function ($row) use ($search_key) {
            foreach ($row as $row_val) {
                if (stripos($row_val, $search_key) !== false) {
                    return true;
                }
            }
        }));

        return $filtered_table_data;

    }

    /**
     * Returns string status label.
     *
     * @param $status_code
     * @return string
     */
    public function get_string_status_label($status_code)
    {

        switch ($status_code) {
            case self::STATUS_NOT_TRANSLATED:
                return __('Not translated', EASY_TRANSLATE);
            case self::STATUS_WAITING:
                return __('Waiting for translator', EASY_TRANSLATE);
            case self::STATUS_PARTIAL:
                return __('Partial translation', EASY_TRANSLATE);
            case self::STATUS_NEEDS_UPDATE:
                return __('Needs update', EASY_TRANSLATE);
            case self::STATUS_COMPLETE:
                return __('Translation complete', EASY_TRANSLATE);
            default:
                return $status_code;
        }

    }

    /**
     * Returns source language.
     *
     * @return string
     */
    public static function get_source_language(): string
    {
        $option = get_option(self::ICL_SITEPRESS_SETTINGS);

        return isset($option['default_language']) ? $option['default_language'] : explode('_', get_locale())[0];
    }

    /**
     * Returns translation languages.
     *
     * @return array
     */
    public static function get_target_languages(): array
    {
        global $wpdb;
        $query = "SELECT code FROM {$wpdb->prefix}icl_languages WHERE active = 1 AND code <> '" . self::get_source_language() . "';";
        $result = $wpdb->get_col($query);
        $langs = [];
        foreach ($result as $lang) {
            $es_lang = get_et_target_language_code($lang);
            if ($es_lang) {
                $langs[] = $lang;
            }
        }

        return $langs;
    }

    /**
     * Returns language names.
     *
     * @param array $codes
     * @return array|object|null
     */
    public static function get_language_name($codes)
    {
        global $wpdb;
        $code_in = '"' . implode('","', $codes) . '"';
        $query = "SELECT english_name, code FROM {$wpdb->prefix}icl_languages WHERE code IN ({$code_in});";

        return $wpdb->get_results($query, ARRAY_A);
    }

    /**
     * Returns true if the translation functionality is ready to run.
     *
     * @return bool
     */
    public static function check_dependencies()
    {
        return is_plugin_active('wpml-string-translation/plugin.php')
               && is_plugin_active('sitepress-multilingual-cms/sitepress.php')
               && get_et_source_language_code(self::get_source_language());
    }

    /**
     * Returns the dependency message list.
     *
     * @return array
     */
    protected function get_dependency_message_list()
    {
        $dependency_message_list = [];

        if (!is_plugin_active('wpml-string-translation/plugin.php')) {
            $dependency_message_list[] = __('Please install and activate WPML String Translation plugin',
                EASY_TRANSLATE);
        }

        if (!is_plugin_active('sitepress-multilingual-cms/sitepress.php')) {
            $dependency_message_list[] = __('Please install and activate WPML Multilingual CMS plugin', EASY_TRANSLATE);
        }

        if (!is_plugin_active('wpml-translation-management/plugin.php')) {
            $dependency_message_list[] = __('Please install and activate WPML Translation Management plugin',
                EASY_TRANSLATE);
        }

        if (!get_et_source_language_code(self::get_source_language())) {
            $dependency_message_list[] = __('Unfortunately EasyTranslate does not support ',
                    EASY_TRANSLATE) . self::get_language_name([self::get_source_language()])[0]['english_name'];
        }

        return $dependency_message_list;
    }

    /**
     * Returns the current admin language option.
     *
     * @return mixed|string
     */
    protected function get_current_admin_language()
    {
        //todo: need refactor or update functionality
        return $this->source_language;

        if (isset($_GET['lang']) && $_GET['lang'] != '') {
            return $_GET['lang'];
        }

        foreach ($_COOKIE as $key => $value) {
            if (stripos($key, '_icl_current_admin_language') !== false) {
                return $value;
            }
        }

        return $this->source_language;
    }

    /**
     * Update string status.
     *
     * @param array $ids
     * @param int   $status
     * @return bool|int
     */
    public static function update_string_status(array $ids, $status)
    {
        global $wpdb;
        $id_in = '"' . implode('","', $ids) . '"';
        $status = intval($status);
        $query = "UPDATE {$wpdb->prefix}icl_strings SET status = {$status} WHERE id IN ({$id_in})";

        return $wpdb->query($query);
    }

    /**
     * Update duplicate post status.
     *
     * @param array $ids
     * @param array $langs
     * @param int   $status
     * @throws Exception
     */
    public static function update_duplicate_status(array $ids, array $langs, int $status)
    {
        foreach ($ids as $post_id) {

            if ($source_translations = self::get_translations_by_element_id($post_id)) {
                foreach ($langs as $lang) {

                    if ($translations = self::get_translations_by_source($source_translations, $lang)) {
                        self::update_translations_status($translations['translation_id'], $status);
                    } else {
                        self::duplicate_post($post_id, $lang);
                        if ($translations = self::get_translations_by_source($source_translations, $lang)) {
                            self::update_translations_status($translations['translation_id'], $status);
                        }
                    }
                }
            }
        }
    }

    /**
     * Duplicate post.
     *
     * @param $post_id
     * @param $lang
     * @throws Exception
     */
    protected static function duplicate_post($post_id, $lang)
    {
        /** @var SitePress $sitepress */
        global $sitepress;

        if ($sitepress->make_duplicate($post_id, $lang) === false) {
            throw new Exception('Unable to duplicate post');
        }
    }

    /**
     * Returns string values by id.
     *
     * @param array $ids
     * @return array
     */
    public function get_strings_content($ids,  string $source_language = null)
    {
        global $wpdb;
        $content = [];
        $id_in = '"' . implode('","', $ids) . '"';
//        $query = "SELECT id, value from {$wpdb->prefix}icl_strings WHERE id IN ({$id_in})";

        $select = "str.id, str.value";
        $where = "str.id IN ({$id_in})";
        $join = '';
        if (!empty($source_language) && $source_language !== self::get_source_language()) {
            $select = "str.id, stran.value as value";
            $where .= " AND stran.language = '{$source_language}'";
            $join = "join wp_icl_string_translations stran on str.id = stran.string_id";
        }
        $query = "SELECT {$select} from {$wpdb->prefix}icl_strings as str {$join} WHERE {$where}";

        $result = $wpdb->get_results($query, ARRAY_A);
        foreach ($result as $item) {
            $content['strings'][$item['id']] = $item['value'];
        }

        return $content;
    }

    /**
     * Die when the nonce check fails.
     */
    public function invalid_nonce_redirect()
    {
        wp_die(__('Invalid Nonce', EASY_TRANSLATE),
            __('Error', EASY_TRANSLATE),
            [
                'response'  => 403,
                'back_link' => true,
            ]
        );

    }

    /**
     * Defines the page template.
     */
    public function render()
    {

        if ($this->is_ready) {
            $this->render_list();
        } else {
            $this->render_requirements();
        }

    }

    /**
     * Render the requirements page.
     */
    protected function render_requirements()
    {

        include_once(EASY_TRANSLATE_ROOT . 'admin/partials/requirements.php');

    }

    /**
     * Returns translations for post.
     *
     * @param $post_id
     * @return array|null
     */
    public static function get_translations_by_element_id($post_id)
    {

        global $wpdb;
        $query = "SELECT * FROM {$wpdb->prefix}icl_translations WHERE element_id={$post_id} AND element_type LIKE 'post_%' LIMIT 1;";

        return $wpdb->get_row($query, ARRAY_A);

    }

    /**
     * Returns translations by source translations.
     *
     * @param array  $source_translations
     * @param string $target_language
     * @return array|object|void|null
     */
    public static function get_translations_by_source($source_translations, $target_language)
    {

        global $wpdb;
        $query = "SELECT * FROM {$wpdb->prefix}icl_translations WHERE trid={$source_translations['trid']} AND language_code='{$target_language}' AND element_type='{$source_translations['element_type']}' LIMIT 1;";

        return $wpdb->get_row($query, ARRAY_A);

    }

    /**
     * Updates post content and title.
     *
     * @param int    $source_post_id
     * @param int    $post_id
     * @param string $target_language
     * @param string $post_content
     * @param string $post_title
     * @param string $post_excerpt
     * @param array  $woocommerce
     * @param array  $yoast_seo
     */
    public static function update_post(
        int $source_post_id,
        int $post_id,
        string $target_language,
        string $post_content = '',
        string $post_title = '',
        string $post_excerpt = '',
        array $woocommerce = [],
        array $yoast_seo = []
    ) {
        $update = [];
        if ($post_title) {
            $update[] = 'post_title="' . esc_sql($post_title) . '"';
        }
        if ($post_content) {
            $update[] = 'post_content="' . esc_sql($post_content) . '"';
        }
        if ($post_excerpt) {
            $update[] = 'post_excerpt="' . esc_sql($post_excerpt) . '"';
        }
        $update = implode(', ', $update);

        global $wpdb;
        $query = "UPDATE {$wpdb->prefix}posts SET {$update} WHERE id = {$post_id}";
        $wpdb->query($query);

        self::update_woocommerce_translations($source_post_id, $post_id, $target_language, $woocommerce);
        self::update_yoast_translations($post_id, $yoast_seo);

        delete_post_meta($post_id, '_icl_lang_duplicate_of', $source_post_id);
    }

    public static function update_yoast_translations(string $post_id, array $yoast_ceo)
    {
        if (empty($yoast_ceo)) {
            return;
        }

        foreach ($yoast_ceo as $key => $translation) {
            update_post_meta($post_id, $key, $translation);
        }
    }

    public static function update_woocommerce_translations(
        string $source_post_id,
        string $post_id,
        string $target_language,
        array $woocommerce
    ) {
        // WooCommerce WPML bridge is not installed
        if (empty($woocommerce) || !class_exists('woocommerce_wpml')) {
            return;
        }

        $translations = [];
        foreach (($woocommerce['_product_attributes'] ?? []) as $key => $value) {
            $translations[md5($key)] = $value['value'];
            $translations[md5("{$key}_name")] = $value['name'];
        }
        if (isset($woocommerce['_purchase_note'])) {
            $translations[md5('_purchase_note')] = $woocommerce['_purchase_note'];
        }
        foreach (($woocommerce['_variations'] ?? []) as $variation_id => $value) {
            $translations[md5("_variation_description{$variation_id}")] = $value['description'];
        }

        $woo_wpml = new woocommerce_wpml();
        $woo_wpml->init();
        do_action('wcml_before_sync_product_data', $source_post_id, $post_id, $target_language);

        $woo_wpml->sync_product_data->duplicate_product_post_meta($source_post_id, $post_id, $translations);
        $woo_wpml->sync_product_data->sync_product_taxonomies($source_post_id, $post_id, $target_language);
        $woo_wpml->attributes->sync_product_attr($source_post_id, $post_id, $target_language, $translations);
        $woo_wpml->attributes->sync_default_product_attr($source_post_id, $post_id, $target_language);
        $woo_wpml->sync_variations_data->sync_product_variations(
            $source_post_id,
            $post_id,
            $target_language,
            [
                'editor_translations' => $translations,
                'is_troubleshooting'  => true,
            ]
        );
        $woo_wpml->sync_product_data->sync_linked_products($source_post_id, $post_id, $target_language);
        $woo_wpml->sync_product_data->sync_product_stock(wc_get_product($source_post_id), wc_get_product($post_id));
        $woo_wpml->media->sync_thumbnail_id($source_post_id, $post_id, $target_language);
    }

    /**
     * Updates post title.
     *
     * @param $post_id
     * @param $post_title
     * @return bool|int
     */
    public static function update_post_title($post_id, $post_title)
    {

        global $wpdb;
        $post_title = esc_sql($post_title);
        $post_id = intval($post_id);
        $query = "UPDATE {$wpdb->prefix}posts SET post_title = '{$post_title}' WHERE id = {$post_id}";

        return $wpdb->query($query);

    }

    /**
     * Update translations status.
     *
     * @param $translation_id
     * @param $status
     */
    public static function update_translations_status($translation_id, $status)
    {
        global $wpdb;
        $wpdb->update("{$wpdb->prefix}icl_translation_status",
            ['status' => $status],
            ['translation_id' => $translation_id],
            ['%d'],
            ['%d']
        );
    }

    /**
     * Returns translation status for main post.
     *
     * @param $post_id
     * @return array
     */
    protected function get_translation_status($post_id)
    {

        global $wpdb;
        $post_id = intval($post_id);
        $query = "SELECT tr.language_code, st.`status`, st.needs_update FROM {$wpdb->prefix}icl_translations AS tr LEFT JOIN {$wpdb->prefix}icl_translation_status AS st ON tr.translation_id=st.translation_id WHERE tr.trid=(SELECT trid FROM {$wpdb->prefix}icl_translations AS tr WHERE tr.element_id={$post_id} AND tr.element_type LIKE 'post_%') AND tr.element_id <> {$post_id};";
        $result = $wpdb->get_results($query, ARRAY_A);

        $translation_status = [];
        foreach ($result as $item) {
            $lang = $item['language_code'];
            $translation_status[$lang] = $item['needs_update'] && $item['status'] != self::STATUS_WAITING ? self::STATUS_NEEDS_UPDATE : $item['status'];
        }

        return $translation_status;

    }

    /**
     * Returns post languages that need to be updated.
     *
     * @param $post_id
     * @return array
     */
    public static function get_translation_needs_update($post_id)
    {

        global $wpdb;
        $query = "SELECT tr.language_code FROM {$wpdb->prefix}icl_translations AS tr LEFT JOIN {$wpdb->prefix}icl_translation_status AS st ON tr.translation_id=st.translation_id WHERE tr.trid=(SELECT trid FROM {$wpdb->prefix}icl_translations AS tr WHERE tr.element_id={$post_id} AND tr.element_type LIKE 'post_%') AND tr.element_id <> {$post_id};";

        //todo: AND st.needs_update=1
        return $wpdb->get_col($query);

    }

    /**
     * Returns post package id.
     *
     * @param $post_id
     * @return string|null
     */
    public static function get_post_package_id($post_id)
    {

        global $wpdb;
        $query = "SELECT ID from {$wpdb->prefix}icl_string_packages WHERE post_id={$post_id} LIMIT 1;";

        return $wpdb->get_var($query);

    }

    /**
     * Return post content.
     *
     * @param int   $post_id
     * @param array $a_string_id
     * @param bool  $updated
     * @return array
     */
    public static function get_post_content(int $post_id, array &$a_string_id, $updated = false): array
    {
        $content = [];
        $package_id = self::get_post_package_id($post_id);
        $post = get_post($post_id);

        $source_language = esc_sql($_REQUEST['source_language'] ?? null);

        // Handle source translation to be actual translation in another language
        if (!empty($source_language) && $source_language !== self::get_source_language()) {
            $source_translations = Easy_Translate_Post_Translation_List::get_translations_by_element_id($post_id);
            $translations = Easy_Translate_Post_Translation_List::get_translations_by_source(
                $source_translations,
                $source_language
            );

            if ($translations['element_id'] ?? null) {
                $post = get_post($translations['element_id']);
            }
        }

        if ($package_id) {
            $strings = self::get_strings_by_package_id($package_id, $updated, $source_language);
            foreach ($strings as $string) {
                $string_id = $string['id'];
                $content['post_strings'][$post_id]['content'][$string_id] = $string['value'];
                $a_string_id[] = $string['id'];
            }
        }
        if ($post) {
            if ($post->post_content && !$package_id) {
                $content['post_full'][$post_id]['content'] = $post->post_content;
            }
            if ($post->post_title) {
                $content['post_full'][$post_id]['title'] = $post->post_title;
            }
            if ($post->post_excerpt) {
                $content['post_full'][$post_id]['excerpt'] = $post->post_excerpt;
            }
            $post_meta_fields = get_post_meta($post_id);

            foreach (self::YOAST_SEO_FIELDS as $yoast_field_name) {
                if (isset($post_meta_fields[$yoast_field_name][0]) && !empty($post_meta_fields[$yoast_field_name][0])) {
                    $content['post_full'][$post_id]['yoast_seo'][$yoast_field_name] = $post_meta_fields[$yoast_field_name][0];
                }
            }

            if (isset($post_meta_fields['_product_attributes']) && !empty($product_attributes = $post_meta_fields['_product_attributes'])) {
                foreach (unserialize($product_attributes[0]) as $field_name => $values) {
                    $content['post_full'][$post_id]['woocommerce']['_product_attributes'][$field_name]['name'] = $values['name'];
                    $content['post_full'][$post_id]['woocommerce']['_product_attributes'][$field_name]['value'] = $values['value'];
                }
            }

            if (isset($post_meta_fields['_purchase_note'][0]) && !empty($purchase_note = $post_meta_fields['_purchase_note'][0])) {
                $content['post_full'][$post_id]['woocommerce']['_purchase_note'] = $purchase_note[0];
            }

            // WooCommerce WPML bridge is not installed
            if (class_exists('woocommerce_wpml')) {
                $woo_wpml = new woocommerce_wpml();
                $woo_wpml->init();

                foreach ($woo_wpml->sync_variations_data->get_product_variations($post_id) as $variation) {
                    $description = get_post_meta($variation->ID, '_variation_description', true);
                    if (!empty($description)) {
                        $content['post_full'][$post_id]['woocommerce']['_variations'][$variation->ID]['description'] = $description;
                    }
                }
            }
        }

        return $content;
    }

    /**
     * Returns translation string array.
     *
     * @param string $package_id
     * @param bool   $updated
     * @return array|object|null
     */
    protected static function get_strings_by_package_id($package_id, $updated = false, $source_language = null)
    {
        global $wpdb;
        $package_id = esc_sql($package_id);
//        $query = "SELECT id, value FROM {$wpdb->prefix}icl_strings WHERE string_package_id='{$package_id}';";
        $select = "str.id, str.value";
        $where = "str.string_package_id='{$package_id}'";
        $join = '';
        if (!empty($source_language) && $source_language !== self::get_source_language()) {
            $select = "str.id, stran.value as value";
            $where .= " AND stran.language = '{$source_language}'";
            $join = "join wp_icl_string_translations stran on str.id = stran.string_id";
        }

        $query = "SELECT {$select} FROM {$wpdb->prefix}icl_strings as str {$join} WHERE {$where};";

        return $wpdb->get_results($query, ARRAY_A);
    }

    /**
     * Returns html with translation language flag.
     *
     * @param array $languages
     * @return string
     */
    protected static function get_translation_language_flag($languages)
    {

        $html = '';
        foreach ($languages as $language) {
            $html .= '<span title="' . $language . '">';
            if ($language == 'zh-hans' || $language == 'zh-hant') {
                $language = 'zh';
            }
            $html .= '<img src="' . plugins_url() . '/sitepress-multilingual-cms/res/flags/' . $language . '.png" width="16" height="12" alt="' . $language . '">';
            $html .= '</span>';
        }

        return $html;

    }

    /**
     * Returns translation_id if post with current source language exists.
     *
     * @param $post_id
     * @return string|null
     */
    public static function compare_post_source_language($post_id)
    {

        global $wpdb;
        $query = "SELECT translation_id from {$wpdb->prefix}icl_translations WHERE element_id={$post_id} AND element_type LIKE 'post_%' AND language_code='" . self::get_source_language() . "' LIMIT 1;";

        return $wpdb->get_var($query);

    }

    /**
     * Set post auto translate value.
     */
    public static function set_auto_translate()
    {

        update_post_meta(intval($_POST['id']), 'et_auto_translate', wp_slash(intval($_POST['value'])));

        wp_die();

    }

    /**
     * Render the list table.
     */
    abstract protected function render_list();

    /**
     * Returns per page option.
     *
     * @return string
     */
    abstract protected function get_per_page_option();

}
