<?php

/**
 * The core plugin class.
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * @package    Easy_Translate
 * @subpackage Easy_Translate/includes
 */
class Easy_Translate
{

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @var      Easy_Translate_Loader $loader Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * Define the core functionality of the plugin.
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     */
    public function __construct()
    {
        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
    }

    /**
     * Load the required dependencies for this plugin.
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     */
    private function load_dependencies()
    {

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once EASY_TRANSLATE_ROOT . 'includes/class-easy-translate-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once EASY_TRANSLATE_ROOT . 'includes/class-easy-translate-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once EASY_TRANSLATE_ROOT . 'admin/class-easy-translate-admin.php';

        /**
         * The class responsible for integration functionality of the plugin.
         */
        require_once EASY_TRANSLATE_ROOT . 'admin/class-easy-translate-api-integration.php';

        /**
         * The class responsible for EasyTranslate API service.
         */
        require_once EASY_TRANSLATE_ROOT . 'admin/class-easy-translate-api-service.php';
        require_once EASY_TRANSLATE_ROOT . 'admin/class-easy-translate-api-laas.php';
        require_once EASY_TRANSLATE_ROOT . 'admin/class-easy-translate-api-string-library.php';

        /**
         * Base class for displaying a list of items in an ajaxified HTML table.
         */
        require_once ABSPATH . '/wp-admin/includes/class-wp-list-table.php';

        /**
         * The class responsible for defining translation list table.
         */
        require_once EASY_TRANSLATE_ROOT . 'admin/class-easy-translate-translation-list.php';

        /**
         * The class responsible for defining string translation list table.
         */
        require_once EASY_TRANSLATE_ROOT . 'admin/class-easy-translate-string-translation-list.php';

        /**
         * The class responsible for defining post translation list table.
         */
        require_once EASY_TRANSLATE_ROOT . 'admin/class-easy-translate-post-translation-list.php';

        /**
         * The class responsible for defining API translation functionality of the plugin.
         */
        require_once EASY_TRANSLATE_ROOT . 'admin/class-easy-translate-translation.php';

        /**
         * The class responsible for defining Projects list.
         */
        require_once EASY_TRANSLATE_ROOT . 'admin/class-easy-translate-projects.php';

        /**
         * The class responsible for updating an existing translation.
         */
        require_once EASY_TRANSLATE_ROOT . 'admin/class-easy-translate-translation-updater.php';

        $this->loader = new Easy_Translate_Loader();

    }

    /**
     * Define the locale for this plugin for internationalization.
     * Uses the Easy_Translate_i18n class in order to set the domain and to register the hook
     * with WordPress.
     */
    private function set_locale()
    {

        $plugin_i18n = new Easy_Translate_i18n();

        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');

    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     */
    private function define_admin_hooks()
    {

        $plugin_admin = new Easy_Translate_Admin();

        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');
        $this->loader->add_action('admin_menu', $plugin_admin, 'admin_menu_setup');
        $this->loader->add_action('admin_init', $plugin_admin, 'init_integration');
        $this->loader->add_action('load-toplevel_page_' . Easy_Translate_Admin::MENU_MAIN, $plugin_admin,
            'load_post_translation_list_screen_options');
        $this->loader->add_filter('set-screen-option', $plugin_admin, 'set_post_translation_list_screen_option', 10, 3);
        $this->loader->add_action('load-easytranslate_page_' . Easy_Translate_Admin::MENU_PARTIAL_TRANSLATIONS,
            $plugin_admin, 'load_string_translation_list_screen_options');
        $this->loader->add_filter('set-screen-option', $plugin_admin, 'set_string_translation_list_screen_option', 10,
            3);
        $this->loader->add_action('wp_ajax_nopriv_' . Easy_Translate_API_Service::WEB_HOOK_ACTION, $plugin_admin,
            'completed_task_callback');
        $this->loader->add_action('edit_post', $plugin_admin, 'edit_post_hook', 101);
        $this->loader->add_action('plugin_loaded', $plugin_admin, 'include_plugin_functions');
        $this->loader->add_action('wp_ajax_et_set_auto_translate_action', $plugin_admin, 'set_auto_translate_callback');

    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     */
    public function run()
    {
        $this->loader->run();
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @return    Easy_Translate_Loader    Orchestrates the hooks of the plugin.
     */
    public function get_loader()
    {
        return $this->loader;
    }

}
